#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

/*
usage: ipkg [options...] sub-command [arguments...]
where sub-command is one of:

Package Manipulation:
        update                  Update list of available packages
        upgrade                 Upgrade all installed packages to latest version
        install <pkg>           Download and install <pkg> (and dependencies)
        install <file.ipk>      Install package <file.ipk>
        remove <pkg>            Remove package <pkg>

Informational Commands:
        list                    List available packages and descriptions
        files <pkg>             List all files belonging to <pkg>
        search <file>           Search for a packaging providing <file>
        info [pkg [<field>]]    Display all/some info fields for <pkg> or all
        status [pkg [<field>]]  Display all/some status fields for <pkg> or all
        download <pkg>          Download <pkg> to current directory.\n
Options:
        -f <conf_file>          Use <conf_file> as the ipkg configuration file
        -conf <conf_file>       Default configuration file location
                                is /etc/ipkg.conf
        -d <dest_name>          Use <dest_name> as the the root directory for
        -dest <dest_name>       package installation, removal, upgrading.
                                <dest_name> should be a defined dest name from
                                the configuration file, (but can also be a
                                directory name in a pinch).
        -o <offline_root>       Use <offline_root> as the root directory for
        -offline <offline_root> offline installation of packages, (currenlty
                                offline installation only works natively)
        Force Options (use when ipkg is too smart for its own good):
        -force-depends          Make dependency checks warnings instead of errors
        -force-defaults         Use default options for questions asked by ipkg.
                                (no prompts). Note that this will not prevent
                                package installation scripts from prompting.
        -force-reinstall        Allow ipkg to reinstall a package.

 */

#include <gnome.h>

#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "interface.h"
#include "support.h"

#include "sgipkg.h"

#ifdef LOCKED
static int lock;
#endif

struct input_add inputAdd;

struct select_pkg selectPackage;
GList *packageInfo = NULL;

struct ipkg_conf ipkgconf;

GtkWidget *wss;
GtkWidget *wrc;

static GdkColor color_selected_background = { 0, 0xf0f0, 0xe0e0, 0xe0e0 };
static GdkColor color_selected_foreground = { 0, 0x0000, 0x0000, 0xffff };

GdkColor color_warning_message = { 0, 0xffff, 0x0000, 0x0000 };
GdkColor color_standard_message = { 0, 0x0000, 0x0000, 0x0000 };

char *warning_message[] = {
  N_("Not select package."),
  N_("Not installed."),
  N_("Already installed."),
  N_("ipkg.conf not found."),
  N_("ipkg command not executed."),
};

char *winconf_env_key[] = {
  "winconf_source_name",
  "winconf_dest_name",
  "winconf_proxy_option"
};

char *winconf_data_key[] = {
  "winconf_source_url",
  "winconf_dest_path",
  "winconf_proxy_url"
};

char *winconf_remove_env_label[] = {
  N_("src-name"),
  N_("dest-name"),
  N_("option")
};

char *winconf_remove_data_label[] = {
  N_("src-url"),
  N_("target-path"),
  N_("proxy-url")
};

/*--------------------------
 * 
 *--------------------------*/

extern GtkWidget *WindowMain;
extern GtkWidget *WindowAbout;
extern GtkWidget *WindowRemove;
extern GtkWidget *WindowConf;
extern GtkWidget *WindowConfRemoveCheck;

/*=====================================================================
 * functions
 *=====================================================================*/

/*---------------------------------------------------------------------
 * init
 *---------------------------------------------------------------------*/

void
init_values (int view)
{
#ifdef LOCKED
  init_lock ();
#endif
  set_view_select (view);

  if (check_ipkg ())
    {
      set_package_info ();
      set_status ();
      reset_clist ();
      init_selected_package ();
    }
  else
    {
#ifdef LOCKED
      set_lock ();/* never execute "ipkg" comamnd */
#endif
      clear_message ();
      write_message (_(warning_message[4]), &color_warning_message);
      write_message ("\n", &color_warning_message);
    }
}


void
refresh ()
{
  reset_status (get_view_select ());
  set_selected_package ("");
  reset_clist ();
}

/*--------------------------
 * selectPackage
 *--------------------------*/
void
init_selected_package ()
{
  strcpy (selectPackage.pkg, "");
  selectPackage.status = STAT_UNINSTALLED;
}

void
set_selected_package (char *pk)
{
  strcpy (selectPackage.pkg, pk);
  if (strlen (selectPackage.pkg) == 0)
    selectPackage.status = STAT_NULL;
  else
    selectPackage.status = get_status (pk);
}

char *
get_selected_package ()
{
  return selectPackage.pkg;
}


gboolean
isSelected ()
{
  if (strcmp (selectPackage.pkg, "") != 0)
    return TRUE;
  else
    return FALSE;
}

gboolean
isInstalled ()
{
  if (strcmp (selectPackage.pkg, "") != 0 
      && selectPackage.status == STAT_INSTALLED)
    return TRUE;
  else
    return FALSE;
}

/*--------------------------
 * lock
 *--------------------------*/
#ifdef LOCKED
void
init_lock ()
{
  lock = NOLOCK;
}

void
set_lock ()
{
  lock = LOCK;
}

void
release_lock ()
{
  lock = NOLOCK;
}


gboolean
isLocked ()
{
  if (lock == LOCK)
    return TRUE;
  else
    return FALSE;
}
#endif

/*--------------------------
 * viewSelect
 *--------------------------*/
void
set_view_select (int v)
{
  selectPackage.viewSelect = v;
}

int
get_view_select ()
{
  return selectPackage.viewSelect;
}

/*---------------------------------------------------------------------
 * opration: packageInfo
 *---------------------------------------------------------------------*/

gboolean
check_ipkg ()
{
  struct stat sb;

  if (stat(IPKG_BIN, &sb) < 0)
      return FALSE;

  if (sb.st_mode & 0111) // executable
    return TRUE;
  else
    return FALSE;
}

void
clear_package_info ()
{
  g_list_foreach (packageInfo, (GFunc)g_free, NULL);
  g_list_free (packageInfo);
  packageInfo = NULL;
}

void
set_package_info ()
{
  FILE *fout;
  char buff[BUFF_SIZE];
  char *ptr;
  struct package_info *tmp;

  g_list_free (packageInfo);

  sprintf (buff, "%s info 2>&1", IPKG_BIN);
  fout = popen (buff, "r");

  if (fout)
    {
      
      while (fgets (buff, BUFF_SIZE, fout))
	{
	  ptr = strtok (buff, " ");
	  
	  /* firstline:Package */
	  if (strcmp (ptr, "Package:") == 0)
	    {
	      
	    RET:
	      
	      tmp = (struct package_info *) calloc (1, sizeof (struct package_info));
	      tmp->package = g_strdup (strtok (NULL, " \n"));
	      tmp->status = STAT_UNINSTALLED;
	      while (strtok (NULL, " \n") != NULL);
	      
	      while (fgets (buff, BUFF_SIZE, fout))
		{
		  ptr = strtok (buff, " ");
		  
		  if (strcmp (ptr, "Section:") == 0)
		    {
		      tmp->section = g_strdup (strtok (NULL, " \n"));
		      while (strtok (NULL, " \n") != NULL);
		    }
		  else if (strcmp (ptr, "Version:") == 0)
		    {
		      tmp->version = g_strdup (strtok (NULL, " \n"));
		      while (strtok (NULL, " \n") != NULL);
		    }
		  else if (strcmp (ptr, "Size:") == 0)
		    {
		      tmp->size = g_strdup (strtok (NULL, " \n"));
		      while (strtok (NULL, " \n") != NULL);
		    }
		  else if (strcmp (ptr, "Package:") == 0)
		    {
		      packageInfo = g_list_append (packageInfo, tmp);
		      goto RET;
		    }
		}
	    }
	}
      pclose (fout);
    }
}

void
reset_status (int view)
{
  GList *node;
  struct package_info *tmp;

  set_view_select (view);

  for (node = packageInfo; node; node = node->next)
    {
      tmp = node->data;
      tmp->status = STAT_UNINSTALLED;
    }
  set_status ();
}

void
set_package_status (char *ptr)
{
  GList *node;
  struct package_info *tmp;

  for (node = packageInfo; node; node = node->next)
    {
      tmp = node->data;
      if (strcmp (tmp->package, ptr) == 0)
	{
	  tmp->status = STAT_INSTALLED;
	  break;
	}
    }
}


void
set_status ()
{
  FILE *fout;
  char buff[BUFF_SIZE];
  char *ptr, *ptr2;

  sprintf (buff, "%s status 2>&1", IPKG_BIN);
  fout = popen (buff, "r");
  if (fout)
    {
      while (fgets (buff, BUFF_SIZE, fout))
	{
	  ptr = strtok (buff, " ");
	  if (strcmp (ptr, "Package:") == 0)
	    {
	    RET2:
	      ptr = g_strdup (strtok (NULL, " \n"));
	      
	      while (fgets (buff, BUFF_SIZE, fout))
		{
		  ptr2 = strtok (buff, " \n");
		  if (strcmp (ptr2, "Package:") == 0)
		    {
		      g_free (ptr);
		      goto RET2;
		    }
		  else if (strcmp (ptr2, "Status:") == 0)
		    {
		      while ((ptr2 = strtok (NULL, " \n")) != NULL)
			{
			  if (strcmp (ptr2, "installed") == 0)
			    {
			      set_package_status (ptr);
			      g_free (ptr);
			    }
			}
		      break;
		    }
		}
	    }
	}
      pclose (fout);
    }
}


int
get_status (char *pk)
{
  struct package_info *tmp;
  GList *node;

  for (node = packageInfo; node; node = node->next)
    {
      tmp = node->data;
      if (strcmp (tmp->package, pk) == 0)
	return tmp->status;
    }
  // STAT_NULL̵
  return STAT_NULL;
}

/*---------------------------------------------------------------------
 * opration: clist
 *---------------------------------------------------------------------*/
void
reset_clist ()
{
  GtkWidget *clist;
  struct package_info *tmp;
  GList *node;
  char *item[4];
  int row = 0;

  clist = get_widget (GTK_WIDGET (WindowMain), "clist");

  gtk_clist_clear (GTK_CLIST (clist));

  for (node = packageInfo; node; node = node->next)
    {
      tmp = node->data;
      item[0] = tmp->package;
      item[1] = tmp->version;
      item[2] = tmp->section;
      item[3] = tmp->size;

      if (get_view_select () == VIEW_ALL
	  || (get_view_select () == VIEW_INSTALLED 
	      && tmp->status == STAT_INSTALLED)
	  || (get_view_select () == VIEW_UNINSTALLED
	      && tmp->status == STAT_UNINSTALLED))
	{

	  gtk_clist_append (GTK_CLIST (clist), item);
	  if (tmp->status == STAT_INSTALLED)
	    {
	      gtk_clist_set_background (GTK_CLIST (clist), row,
					&color_selected_background);
	      gtk_clist_set_foreground (GTK_CLIST (clist), row,
					&color_selected_foreground);
	    }
	  row++;
	}
    }
}


/*---------------------------------------------------------------------
 * WindowMain:text
 *---------------------------------------------------------------------*/
void
clear_message ()
{
  GtkWidget *Msg;

  Msg = get_widget (GTK_WIDGET (WindowMain), "Msg");

  gtk_text_freeze (GTK_TEXT (Msg));
  gtk_text_backward_delete (GTK_TEXT (Msg),
			    gtk_text_get_length (GTK_TEXT (Msg)));
  gtk_text_thaw (GTK_TEXT (Msg));
}


void
write_message (char *msg, GdkColor * clr)
{
  GtkWidget *Msg;

  Msg = get_widget (GTK_WIDGET (WindowMain), "Msg");

  gtk_text_freeze (GTK_TEXT (Msg));
  gtk_text_insert (GTK_TEXT (Msg), NULL, clr, NULL, msg, strlen (msg));
  gtk_text_thaw (GTK_TEXT (Msg));
}

/*---------------------------------------------------------------------
 * WindowMain: radiobutton 
 *---------------------------------------------------------------------*/

void
action_radiobutton (int view)
{
  set_view_select (view);
  reset_clist ();
  set_selected_package ("");
  clear_message ();
}

/*---------------------------------------------------------------------
 * WindowRemove
 *---------------------------------------------------------------------*/

void
write_package_name (GtkWidget * WindowRemove)
{
  char string[MBUFF_SIZE];
  GtkWidget *label;

  label = get_widget (GTK_WIDGET (WindowRemove), "RemoveCheck");

  sprintf (string, "%s", get_selected_package ());
  gtk_label_set (GTK_LABEL (label), string);
}



/*---------------------------------------------------------------------
 * do_ipkg(s)
 *---------------------------------------------------------------------*/
void
do_ipkg_nowait (char *command, char *pkg)
{
  char buff[BUFF_SIZE];

#ifdef LOCKED
  set_lock ();
#endif
  sprintf (buff, "%s %s %s 2>&1", IPKG_BIN, command, pkg);
  do_ipkg_and_write_message (buff);
#ifdef LOCKED
  release_lock ();
#endif
}

void
do_ipkg_waiting (char *command, char *pkg, int post_work_type)
{
  char buff[BUFF_SIZE];
  sprintf (buff, "%s %s %s 2>&1", IPKG_BIN, command, pkg);
  do_ipkg_and_waiting (buff, post_work_type);
}


void
do_ipkg_and_write_message (char *command)
{
  FILE *fout;
  GtkWidget *Msg;
  char buff[BUFF_SIZE];
  int len;
  int line = 0;

  fout = popen (command, "r");
  if (!fout)
      gtk_exit (1);

  Msg = get_widget (GTK_WIDGET (WindowMain), "Msg");

  gtk_text_freeze (GTK_TEXT (Msg));
  gtk_text_backward_delete (GTK_TEXT (Msg),
			    gtk_text_get_length (GTK_TEXT (Msg)));
  while (fgets (buff, BUFF_SIZE, fout))
    {
      len = strlen (buff);
      gtk_text_insert (GTK_TEXT (Msg), NULL, NULL, NULL, buff, len);
      line++;
    }
  pclose (fout);

  if (line == 0)
    {
      clear_message ();
      write_message (_(warning_message[1]), &color_standard_message);
      write_message ("\n", &color_warning_message);
    }

  gtk_text_thaw (GTK_TEXT (Msg));
}


void
read_fd (gpointer data, gint fd, GdkInputCondition condition)
{
  char buff[BUFF_SIZE];
  int len;
  int post_work_type = (int) data;

  len = read (fd, buff, sizeof (buff));
  if ((condition && GDK_INPUT_READ) && len > 0)
    {
      gtk_text_insert (GTK_TEXT (inputAdd.widget), 
		       NULL, NULL, NULL, buff, len);
    }
  else
    {
      refresh ();

      gdk_input_remove (inputAdd.tag);
      pclose (inputAdd.fd);

#ifdef LOCKED
      release_lock ();
#endif
      switch (post_work_type)
	{
	case POST_RESET_PKG_INFO:
	  clear_package_info ();
	  if (check_ipkg ())
	    {
	      set_package_info ();
	      set_status ();
	      reset_clist ();
	      init_selected_package ();
	    }
	  break;
	case POST_NONE:
	default:
	  break;
	}
    }
}


void
do_ipkg_and_waiting (char *command, int post_work_type)
{
  FILE *fout;
  GtkWidget *Msg;

  fout = popen (command, "r");
  if (!fout)
      gtk_exit (1);

#ifdef LOCKED
  set_lock ();
#endif

  Msg = get_widget (GTK_WIDGET (WindowMain), "Msg");
  gtk_text_freeze (GTK_TEXT (Msg));
  gtk_text_backward_delete (GTK_TEXT (Msg),
			    gtk_text_get_length (GTK_TEXT (Msg)));
  gtk_text_thaw (GTK_TEXT (Msg));

  inputAdd.widget = Msg;
  inputAdd.fd = fout;
  inputAdd.tag = gdk_input_add (fileno (fout), GDK_INPUT_READ, 
				read_fd, (gpointer)post_work_type);
}


/*--------------------------
 * do_ipkg(s)
 *--------------------------*/
void
do_ipkg_info (char *pkg)
{
  do_ipkg_nowait ("info", pkg);
}

void
do_ipkg_update_list ()
{
  do_ipkg_waiting ("update", "", POST_RESET_PKG_INFO);
}


void
do_ipkg_update (char *pkg)
{
  do_ipkg_waiting ("update", pkg, POST_NONE);
}


void
do_ipkg_upgrade ()
{
  do_ipkg_waiting ("upgrade", "", POST_NONE);
}

void
do_ipkg_install (char *pkg)
{
  do_ipkg_waiting ("install", pkg, POST_NONE);
}

void
do_ipkg_remove (char *pkg)
{
  do_ipkg_nowait ("remove", pkg);
}


void
do_ipkg_search (char *pkg)
{
  do_ipkg_nowait ("search", pkg);
}


void
do_ipkg_status (char *pkg)
{
  do_ipkg_nowait ("status", pkg);
}

void
do_ipkg_depends (char *pkg)
{
  do_ipkg_nowait ("files", pkg);
}

/*---------------------------------------------------------------------
 * configure window
 *---------------------------------------------------------------------*/

#define NOT_SELECT -1

void
init_confs ()
{
  int i;

  for (i = 0; i < 3; i++)
    {
      ipkgconf.row[i] = NOT_SELECT;
      ipkgconf.list[i] = NULL;
    }

  ipkgconf.select_page = SOURCE;
  ipkgconf.conf = read_conf ();

  if (!check_conf_file ())
      write_message (_(warning_message[3]), &color_warning_message);
}


void
init_clist_conf (char *name, GList * list)
{
  GtkWidget *clist;
  GList *node;
  char *item[2];
  struct conf_data *tmp;

  clist = get_widget (GTK_WIDGET (wss), name);
  gtk_clist_clear (GTK_CLIST (clist));
  for (node = list; node; node = node->next)
    {
      tmp = node->data;
      item[0] = tmp->env;
      item[1] = tmp->data;
      gtk_clist_append (GTK_CLIST (clist), item);
    }
}


void
init_window_conf ()
{
  int i;
  char *tmp[]
    = {"winconf_clist_source", "winconf_clist_dest", "winconf_clist_proxy"};

  for (i = 0; i < 3; i++)
    {
      init_clist_conf (tmp[i], ipkgconf.list[i]);
      ipkgconf.row[i] = NOT_SELECT;
    }
}

#undef NOT_SELECT


/*--------------------------
 * commands
 *--------------------------*/

void
do_winconf_add (int item)
{
  GtkWidget *entry_env;
  GtkWidget *entry_data;
  char *env;
  char *data;
  struct conf_data *tmp;

  entry_env = get_widget (GTK_WIDGET (wss), winconf_env_key[item]);
  entry_data = get_widget (GTK_WIDGET (wss), winconf_data_key[item]);

  env = strdup (gtk_entry_get_text (GTK_ENTRY (entry_env)));
  data = strdup (gtk_entry_get_text (GTK_ENTRY (entry_data)));

  if (strlen (env) != 0 && strlen (data) != 0)
    {
      if (check_unique_conf_data (env, data, ipkgconf.list[item]))
	{
	  tmp = (struct conf_data *) calloc (1, sizeof (struct conf_data));
	  tmp->env = env;
	  tmp->data = data;

	  ipkgconf.list[item] = g_list_prepend (ipkgconf.list[item], tmp);

	  write_ipkgconf ();

	  init_window_conf ();
	  clear_winconf_text_entry (ipkgconf.select_page);
	}
    }
  else
    {
      g_free (env);
      g_free (data);
    }
}


void
do_winconf_edit (int item)
{
  GtkWidget *entry_env;
  GtkWidget *entry_data;
  char *env;
  char *data;
  struct conf_data *tmp;
  int row;
  GList *node;

  entry_env = get_widget (GTK_WIDGET (wss), winconf_env_key[item]);
  entry_data = get_widget (GTK_WIDGET (wss), winconf_data_key[item]);

  env = strdup (gtk_entry_get_text (GTK_ENTRY (entry_env)));
  data = strdup (gtk_entry_get_text (GTK_ENTRY (entry_data)));

  if (strlen (env) != 0 && strlen (data) != 0)
    {
      if (check_unique_conf_data (env, data, ipkgconf.list[item]))
	{
	  row = ipkgconf.row[item];

	  node = g_list_nth (ipkgconf.list[item], row);
	  tmp = node->data;
	  tmp->env = env;
	  tmp->data = data;

	  write_ipkgconf ();

	  init_window_conf ();
	  clear_winconf_text_entry (ipkgconf.select_page);
	}
    }
  else
    {
      g_free (env);
      g_free (data);
    }
}


void
do_winconf_delete (int item)
{
  GtkWidget *entry_env;
  GtkWidget *entry_data;
  char *env;
  char *data;

  GtkWidget *WindowConfRemoveCheck;

  entry_env = get_widget (GTK_WIDGET (wss), winconf_env_key[item]);
  entry_data = get_widget (GTK_WIDGET (wss), winconf_data_key[item]);

  env = strdup (gtk_entry_get_text (GTK_ENTRY (entry_env)));
  data = strdup (gtk_entry_get_text (GTK_ENTRY (entry_data)));

  if (strlen (env) != 0 && strlen (data) != 0)
    {
      if (!check_unique_conf_data (env, data, ipkgconf.list[item]))
	{
	  WindowConfRemoveCheck = create_WindowConfRemoveCheck ();

	  wrc = WindowConfRemoveCheck;
	  ipkgconf.select_page = item;
	  //	  write_winconf_remove_env_data (WindowConfRemoveCheck);
	  gtk_widget_show (WindowConfRemoveCheck);
	}
    }
  else
    {
      g_free (env);
      g_free (data);
    }
}


void
do_winconf_remove ()
{
  GList *node;
  int item;

  item = ipkgconf.select_page;

  node = g_list_nth (ipkgconf.list[item], ipkgconf.row[item]);
  ipkgconf.list[item] = g_list_remove (ipkgconf.list[item], node->data);

  write_ipkgconf ();

  init_window_conf ();
  clear_winconf_text_entry (item);
}

/*--------------------------
 * utils
 *--------------------------*/

gboolean
check_unique_conf_data (char *env, char *data, GList * list)
{
  struct conf_data *tmp;
  GList *node;

  for (node = list; node; node = node->next)
    {
      tmp = node->data;
      if (strcmp (env, tmp->env) == 0 && strcmp (data, tmp->data) == 0)
	return FALSE;
    }
  return TRUE;
}


gboolean
check_conf_file ()
{
  return ipkgconf.conf;
}

char *
get_ipkgconf_dir ()
{
  char *s;

  if ((s = getenv ("IPKG_CONF_DIR")) == NULL)
      return IPKG_CONF_DIR;
  else
      return s;
}


#define ENV  0
#define DATA 1

char *
get_winconf_remove_envdata (int key)
{
  GList *node;
  struct conf_data *tmp;
  int item;

  item = ipkgconf.select_page;
  node = g_list_nth (ipkgconf.list[item], ipkgconf.row[item]);
  tmp = node->data;

  if (key == ENV)
    return (char *) tmp->env;
  else
    return (char *) tmp->data;
}


char *
get_winconf_remove_env ()
{
  return get_winconf_remove_envdata (ENV);
}

char *
get_winconf_remove_data ()
{
  return get_winconf_remove_envdata (DATA);
}

#undef ENV
#undef DATA


void
write_winconf_remove_env_data (GtkWidget * WindowConfRemoveCheck)
{
  char buff[MBUFF_SIZE];
  GtkWidget *label;
  int item = ipkgconf.select_page;

  label = get_widget (GTK_WIDGET (WindowConfRemoveCheck),
		      "winconf_remove_env");
  sprintf (buff, "%s:%s",
	   winconf_remove_env_label[item], get_winconf_remove_env ());
  gtk_label_set (GTK_LABEL (label), buff);

  label = get_widget (GTK_WIDGET (WindowConfRemoveCheck),
		      "winconf_remove_data");
  sprintf (buff, "%s:%s",
	   winconf_remove_data_label[item], get_winconf_remove_data ());
  gtk_label_set (GTK_LABEL (label), buff);
}


void
do_winconf_proxy_delete ()
{
  do_winconf_delete (PROXY);
}


void
clear_winconf_text_entry (int num)
{
  int i;

  /* remark:windowConfnotebookڡֹȡselect_pageϰפƤ */
  ipkgconf.select_page = num;

  for (i = 0; i < 3; i++)
    {
      set_winconf_text_entry (winconf_env_key[i], "");
      set_winconf_text_entry (winconf_data_key[i], "");
    }
  set_winconf_text_entry (winconf_env_key[2], "http_proxy");
  set_winconf_text_entry (winconf_data_key[2], "http://");
}


void
set_winconf_text_entry (char *e, char *t)
{
  GtkWidget *entry;
  entry = get_widget (GTK_WIDGET (wss), e);
  gtk_entry_set_text (GTK_ENTRY (entry), t);
}


void
set_winconf_entry (int item, int row)
{
  GList *node;
  struct conf_data *tmp;

  ipkgconf.row[item] = row;
  node = g_list_nth (ipkgconf.list[item], row);
  tmp = node->data;
  set_winconf_text_entry (winconf_env_key[item], (char *) tmp->env);
  set_winconf_text_entry (winconf_data_key[item], (char *) tmp->data);
}


/*--------------------------
 * read/write ipkg.conf
 *--------------------------*/

gboolean
read_conf ()
{
  FILE *fd;
  char buff[BUFF_SIZE];
  char conf_file[SBUFF_SIZE];
  char *conf_dir;
  char *ptr;
  char *env;
  char *data;

  struct conf_data *tmp;

  conf_dir = get_ipkgconf_dir ();
  sprintf (conf_file, "%s/ipkg.conf", conf_dir);

  if ((fd = fopen (conf_file, "r")) != NULL)
    {
      while (fgets (buff, sizeof (buff), fd))
	{
	  if (buff[0] == '\n' || buff[0] == '\t' 
	      || buff[0] == '#' || buff[0] == ' ')
	    {
	      continue;
	    }
	  else
	    {
	      ptr = strtok (buff, " ");

	      if (strcmp (ptr, "src") == 0)
		{
		  env = g_strdup (strtok (NULL, " \n"));
		  data = g_strdup (strtok (NULL, " \n"));
		  while (strtok (NULL, " \n") != NULL);

		  if (strlen (env) != 0 && strlen (data) != 0)
		    {
		      tmp = (struct conf_data *) calloc (1, sizeof (struct conf_data));
		      tmp->env = env;
		      tmp->data = data;
		      ipkgconf.list[0] = g_list_append (ipkgconf.list[0], tmp);
		    }
		  else
		    {
		      g_free (env);
		      g_free (data);
		    }
		}
	      else if (strcmp (ptr, "dest") == 0)
		{
		  env = g_strdup (strtok (NULL, " \n"));
		  data = g_strdup (strtok (NULL, " \n"));
		  while (strtok (NULL, " \n") != NULL);

		  if (strlen (env) != 0 && strlen (data) != 0)
		    {
		      tmp = (struct conf_data *) calloc (1, sizeof (struct conf_data));
		      tmp->env = env;
		      tmp->data = data;
		      ipkgconf.list[1] = g_list_append (ipkgconf.list[1], tmp);
		    }
		  else
		    {
		      g_free (env);
		      g_free (data);
		    }
		}
	      else if (strcmp (ptr, "option") == 0)
		{
		  env = g_strdup (strtok (NULL, " \n"));
		  data = g_strdup (strtok (NULL, " \n"));
		  while (strtok (NULL, " \n") != NULL);

		  if (strlen (env) != 0 && strlen (data) != 0
		      && strstr (env, "proxy") != NULL)
		    {
		      tmp = (struct conf_data *) calloc (1, sizeof (struct conf_data));
		      tmp->env = env;
		      tmp->data = data;
		      ipkgconf.list[2] = g_list_append (ipkgconf.list[2], tmp);
		    }
		  else
		    {
		      g_free (env);
		      g_free (data);
		    }
		}
	    }
	}
      fclose (fd);
    }
  else
    {
      return FALSE;
    }
  return TRUE;
}

void
write_conf (FILE * fd, char *key, GList * list)
{
  GList *node;
  struct conf_data *tmp;

  for (node = list; node; node = node->next)
    {
      tmp = node->data;
      fprintf (fd, "%s %s %s\n", key, (char *) tmp->env, (char *) tmp->data);
    }
}

void
write_ipkgconf ()
{
  FILE *fd;
  char buff[BUFF_SIZE];
  char conf_file[SBUFF_SIZE];
  char *conf_dir;
  char *ptr;
  struct conf_data *tmp;
  char *env;
  GList *node;
  GList *conf = NULL;
  int i;

  char *item[] =  {"src", "dest", "option"};
  gboolean flag[] = {TRUE, TRUE, TRUE};


  conf_dir = get_ipkgconf_dir ();
  sprintf (conf_file, "%s/ipkg.conf", conf_dir);

  /* read ipkg.conf */
  if ((fd = fopen (conf_file, "r")) != NULL)
    {
      while (fgets (buff, sizeof (buff), fd))
	{
	  if (buff[0] == '\n' || buff[0] == '\t'
	      || buff[0] == '#' || buff[0] == ' ')
	    {
	      ptr = g_strdup (buff);
	      conf = g_list_append (conf, ptr);
	    }
	  else
	    {
	      ptr = strtok (buff, " ");

	      if (strcmp (ptr, "src") == 0 && flag[0] == TRUE)
		{
		  ptr = g_strdup (item[0]);
		  conf = g_list_append (conf, ptr);
		  while (strtok (NULL, " \n") != NULL);
		  flag[0] = FALSE;
		}
	      else if (strcmp (ptr, item[1]) == 0 && flag[1] == TRUE)
		{
		  ptr = g_strdup (item[1]);
		  conf = g_list_append (conf, ptr);
		  while (strtok (NULL, " \n") != NULL);
		  flag[1] = FALSE;
		}
	      else if (strcmp (ptr, item[2]) == 0 && flag[2] == TRUE)
		{
		  env = g_strdup (strtok (NULL, " \n"));
		  if (strstr (env, "proxy") != NULL)
		    {
		      ptr = g_strdup (item[2]);
		      conf = g_list_append (conf, ptr);
		      flag[2] = FALSE;
		    }
		  while (strtok (NULL, " \n") != NULL);
		  g_free (env);
		}
	    }
	}
      fclose (fd);
    }

  /* write ipkg.conf */
  flag[0] = flag[1] = flag[2] = TRUE;

  if ((fd = fopen (conf_file, "w")) != NULL)
    {
      for (node = conf; node; node = node->next)
	{
	  tmp = node->data;

	  if (strcmp ((char *) tmp, item[0]) == 0 && flag[0] == TRUE)
	    {
	      write_conf (fd, item[0], ipkgconf.list[0]);
	      flag[0] = FALSE;
	    }
	  else if (strcmp ((char *) tmp, item[1]) == 0 && flag[1] == TRUE)
	    {
	      write_conf (fd, item[1], ipkgconf.list[1]);
	      flag[1] = FALSE;
	    }
	  else if (strcmp ((char *) tmp, item[2]) == 0 && flag[2] == TRUE)
	    {
	      write_conf (fd, item[2], ipkgconf.list[2]);
	      flag[2] = FALSE;
	    }
	  else
	    {
	      fprintf (fd, "%s", (char *) tmp);
	    }
	}

      for (i = 0; i < 3; i++)
	  if (flag[i] == TRUE)
	    write_conf (fd, item[i], ipkgconf.list[i]);

      fclose (fd);
    }

  g_list_foreach (conf, (GFunc)g_free, NULL);
  g_list_free (conf);

}

/* EOF */
